#include "opencv2/core/core.hpp"
#include "opencv2/aruco.hpp"
#include "opencv2/highgui.hpp"
#include <iostream>
#include <string>
#include <map>

int main(int argc, char* argv[]) {
    // Comprovem si hi ha prou paràmetres en la línia de comandes
    if (argc != 5) {
        std::cerr << "Usage: " << argv[0] << " <dictionary> <id> <size> <output_file>" << std::endl;
        return -1;
    }

    // Paràmetres de la línia de comandes
    std::string dictionaryName = argv[1];    // Nom del diccionari (per exemple, DICT_4X4_50)
    int markerId = std::atoi(argv[2]);       // ID de la marca (numèric)
    int markerSize = std::atoi(argv[3]);     // Mida de la marca en píxels
    std::string outputFile = argv[4];        // Nom del fitxer de sortida (per exemple, marker.png)

    // Mapa de diccionaris possibles
    std::map<std::string, cv::Ptr<cv::aruco::Dictionary>> dictionaryMap = {
        {"DICT_4X4_50", cv::aruco::getPredefinedDictionary(cv::aruco::DICT_4X4_50)},
        {"DICT_5X5_100", cv::aruco::getPredefinedDictionary(cv::aruco::DICT_5X5_100)},
        {"DICT_6X6_250", cv::aruco::getPredefinedDictionary(cv::aruco::DICT_6X6_250)},
        {"DICT_7X7_50", cv::aruco::getPredefinedDictionary(cv::aruco::DICT_7X7_50)},
        {"DICT_4X4_100", cv::aruco::getPredefinedDictionary(cv::aruco::DICT_4X4_100)},
        // Afegir més diccionaris segons les necessitats
    };

    // Comprovem si el diccionari passat existeix en el mapa
    if (dictionaryMap.find(dictionaryName) == dictionaryMap.end()) {
        std::cerr << "Error: Diccionari no suportat o incorrecte!" << std::endl;
        return -1;
    }

    // Obtenir el diccionari seleccionat
    cv::Ptr<cv::aruco::Dictionary> dictionary = dictionaryMap[dictionaryName];

    // Crear la matriu que contendrà el marker
    cv::Mat markerImage;

    // Dibuixar el marker amb el diccionari i l'ID especificats
    cv::aruco::drawMarker(dictionary, markerId, markerSize, markerImage);

    // Afegir un marc blanc al voltant del marker
    int borderThickness = 50; // Mida del marc blanc
    cv::Mat markerWithBorder(markerSize + 2 * borderThickness, markerSize + 2 * borderThickness, markerImage.type(), cv::Scalar(255, 255, 255));

    // Copiar el marker dins de la matriu amb el marc blanc
    markerImage.copyTo(markerWithBorder(cv::Rect(borderThickness, borderThickness, markerSize, markerSize)));

    // Desar la imatge en format PNG
    if (!cv::imwrite(outputFile, markerWithBorder)) {
        std::cerr << "Error: No s'ha pogut desar la imatge!" << std::endl;
        return -1;
    }

    std::cout << "Marker generat correctament amb marc blanc i desat a: " << outputFile << std::endl;

    return 0;
}

