#include "opencv2/core/core.hpp"
#include "opencv2/imgproc.hpp"
#include "opencv2/highgui.hpp"
#include "opencv2/aruco.hpp"   // Importar la biblioteca de ArUco

#include <iostream>
#include <map>  // Afegir capçalera per a std::map

int main(int argc, char* argv[]) {
  if (argc < 3){
    std::cerr << "Usage: " << argv[0] << " <video source no.> <dictionary>" << std::endl;
    return -1;
  }

  cv::VideoCapture webCam(std::atoi(argv[1]));       // VideoCapture object declaration. Usually 0 is the integrated, 2 is the first external USB one
  std::string dictionaryName = argv[2];    // Nom del diccionari (per exemple, DICT_4X4_50)

  if (webCam.isOpened() == false){   // Check if the VideoCapture object has been correctly associated to the webcam
    std::cerr << "error: Webcam could not be connected." << std::endl;
    return -1;
  }

  cv::Mat imgOriginal;    // input image
  char charCheckForESCKey{0};

  // Inicialitzar el diccionari ArUco

  // Mapa de diccionaris possibles
  std::map<std::string, cv::Ptr<cv::aruco::Dictionary>> dictionaryMap;

  dictionaryMap["DICT_4X4_50"] = cv::aruco::getPredefinedDictionary(cv::aruco::DICT_4X4_50);
  dictionaryMap["DICT_5X5_100"] = cv::aruco::getPredefinedDictionary(cv::aruco::DICT_5X5_100);
  dictionaryMap["DICT_6X6_250"] = cv::aruco::getPredefinedDictionary(cv::aruco::DICT_6X6_250);
  dictionaryMap["DICT_7X7_50"] = cv::aruco::getPredefinedDictionary(cv::aruco::DICT_7X7_50);
  dictionaryMap["DICT_4X4_100"] = cv::aruco::getPredefinedDictionary(cv::aruco::DICT_4X4_100);

  // Comprovem si el diccionari passat existeix en el mapa
  if (dictionaryMap.find(dictionaryName) == dictionaryMap.end()) {
    std::cerr << "Error: Diccionari no suportat o incorrecte!" << std::endl;
    return -1;
  }

  // Obtenir el diccionari seleccionat
  cv::Ptr<cv::aruco::Dictionary> dictionary = dictionaryMap[dictionaryName];
  
  std::vector<int> ids;    // Vectors per emmagatzemar IDs dels ArUcos detectats
  std::vector<std::vector<cv::Point2f>> corners;    // Vectors per emmagatzemar les cantonades detectades

  while (charCheckForESCKey != 27 && webCam.isOpened()){    // loop until ESC key is pressed or webcam is lost
    bool frameSuccess = webCam.read(imgOriginal);           // get next frame from input stream

    if (!frameSuccess || imgOriginal.empty()){              // if the frame was not read or read wrongly
      std::cerr << "error: Frame could not be read." << std::endl;
      break;
    }

    // Detectar els codis ArUco
    cv::aruco::detectMarkers(imgOriginal, dictionary, corners, ids);

    // Si es detecten codis ArUco, es dibuixen a la imatge
    if (ids.size() > 0) {
      cv::aruco::drawDetectedMarkers(imgOriginal, corners, ids);
    }

    // Mostrar la imatge
    cv::imshow("imgOriginal", imgOriginal);

    // Esperar per una tecla, sortir si es prem ESC
    charCheckForESCKey = cv::waitKey(1);  // gets the key pressed
  }
  return 0;
}

